@@ -47,21 +47,34 @@ class Drummer:
         else:
             self.kit[name] = { 'instrument': patch, 'part': stream.Part(), 'counter': 0 }
 
+    def _is_hihat(self, item):
+        return re.search(r"^hihat\d$", item) or item == 'closed' or item == 'open' or item == 'pedal'
+    def _is_kick(self, item):
+        return re.search(r"^kick\d$", item)
+    def _is_snare(self, item):
+        return re.search(r"^snare\d$", item) or item == 'sidestick'
+    def _is_tom(self, item):
+        return re.search(r"^tom\d$", item)
+    def _is_cymbal(self, item):
+        return re.search(r"^ride.+$", item) or re.search(r"^crash\d$", item) or item == 'china' or item == 'splash'
+    def _is_percussion(self, item):
+        return re.search(r"^bongo\d$", item) or re.search(r"^conga\d$", item) or re.search(r"^timbale\d$", item) or re.search(r"^woodblock\d$", item) or re.search(r"^triangle\d$", item) or item == 'clap' or item == 'cowbell' or item == 'tambourine' or item == 'shaker' or item == 'cabasa' or item == 'claves'
+
     def rest(self, name, duration=1.0):
         if not isinstance(name, list):
             name = [name]
         for item in name:
-            if item == 'closed':
+            if self._is_hihat(item):
                 item = 'hihat'
-            elif item == 'open':
-                item = 'hihat'
-            elif item == 'pedal':
-                item = 'hihat'
-            elif re.search(r"^tom\d$", item):
+            elif self._is_kick(item):
+                item = 'kick'
+            elif self._is_snare(item):
+                item = 'snare'
+            elif self._is_tom(item):
                 item = 'toms'
-            elif re.search(r"^ride.+$", item) or re.search(r"^crash\d$", item) or item == 'china' or item == 'splash':
+            elif self._is_cymbal(item):
                 item = 'cymbals'
-            elif re.search(r"^bongo\d$", item) or re.search(r"^conga\d$", item) or re.search(r"^timbale\d$", item) or re.search(r"^woodblock\d$", item) or re.search(r"^triangle\d$", item) or item == 'clap' or item == 'cowbell' or item == 'tambourine' or item == 'shaker' or item == 'cabasa' or item == 'claves':
+            elif self._is_percussion(item):
                 item = 'percussion'
             n = note.Rest()
             n.duration = m21duration.Duration(duration)
@@ -69,38 +82,47 @@ class Drummer:
             self.kit[item]['counter'] += duration
 
     def note(self, name, duration=1.0, volume=None, flam=0):
-        if name == 'hihat' or name == 'closed':
-            patch = self.instrument_map('hihat1')
-            name = 'hihat'
-        elif name == 'open':
-            patch = self.instrument_map('hihat2')
-            name = 'hihat'
-        elif name == 'pedal':
-            patch = self.instrument_map('hihat3')
-            name = 'hihat'
-        elif re.search(r"^tom\d$", name):
-            patch = self.instrument_map(name)
-            name = 'toms'
-        elif re.search(r"^ride.+$", name) or re.search(r"^crash\d$", name) or name == 'china' or name == 'splash':
-            patch = self.instrument_map(name)
-            name = 'cymbals'
-        elif re.search(r"^bongo\d$", name) or re.search(r"^conga\d$", name) or re.search(r"^timbale\d$", name) or re.search(r"^woodblock\d$", name) or re.search(r"^triangle\d$", name) or name == 'clap' or name == 'cowbell' or name == 'tambourine' or name == 'shaker' or name == 'cabasa' or name == 'claves':
-            patch = self.instrument_map(name)
-            name = 'percussion'
-        else:
-            patch = self.instrument_map(self.kit[name]['instrument'])
-        if volume is None:
-            volume = self.volume
-        if flam > 0:
-            grace = note.Note(patch['num'])
-            grace.volume.velocity = volume
-            grace.duration = m21duration.Duration(flam)
-            self.kit[name]['part'].append(grace)
-        n = note.Note(patch['num'])
-        n.volume.velocity = volume
-        n.duration = m21duration.Duration(duration - flam)
-        self.kit[name]['part'].append(n)
-        self.kit[name]['counter'] += duration
+        if not isinstance(name, list):
+            name = [name]
+        for item in name:
+            if item == 'hihat' or item == 'closed':
+                patch = self.instrument_map('hihat1')
+                item = 'hihat'
+            elif item == 'open':
+                patch = self.instrument_map('hihat2')
+                item = 'hihat'
+            elif item == 'pedal':
+                patch = self.instrument_map('hihat3')
+                item = 'hihat'
+            elif self._is_kick(item):
+                patch = self.instrument_map(item)
+                item = 'kick'
+            elif self._is_snare(item):
+                patch = self.instrument_map(item)
+                item = 'snare'
+            elif self._is_tom(item):
+                patch = self.instrument_map(item)
+                item = 'toms'
+            elif self._is_cymbal(item):
+                patch = self.instrument_map(item)
+                item = 'cymbals'
+            elif self._is_percussion(item):
+                patch = self.instrument_map(item)
+                item = 'percussion'
+            else:
+                patch = self.instrument_map(self.kit[item]['instrument'])
+            if volume is None:
+                volume = self.volume
+            if flam > 0:
+                grace = note.Note(patch['num'])
+                grace.volume.velocity = volume
+                grace.duration = m21duration.Duration(flam)
+                self.kit[item]['part'].append(grace)
+            n = note.Note(patch['num'])
+            n.volume.velocity = volume
+            n.duration = m21duration.Duration(duration - flam)
+            self.kit[item]['part'].append(n)
+            self.kit[item]['counter'] += duration
 
     def accent_note(self, name, duration=1.0, volume=None, accent=None):
         if volume is None:
@@ -132,8 +154,12 @@ class Drummer:
                     for bit in pattern_str:
                         if bit == '0':
                             self.rest(inst, duration=duration)
-                        else:
+                        elif bit == '1':
                             self.note(inst, duration=duration, volume=volume)
+                        elif bit == '2':
+                            self.note('open', duration=duration, volume=volume)
+                        elif bit == '3':
+                            self.note('pedal', duration=duration, volume=volume)
 
     def roll(self, name, duration=1, subdivisions=4, crescendo=[]):
         if not crescendo:
@@ -153,14 +179,14 @@ class Drummer:
         # n.b. format=None shows whatever the music21 default is set to
         self.score.show(format)
 
-    def instrument_map(self, key=None):
+    def instrument_map(self, key=None, num=None, name=None):
         kit = {
             'kick1': { 'num': 35, 'name': 'Acoustic Bass Drum', 'obj': instrument.BassDrum() },
             'kick2': { 'num': 36, 'name': 'Bass Drum 1', 'obj': instrument.BassDrum() },
             'snare1': { 'num': 38, 'name': 'Acoustic Snare', 'obj': instrument.SnareDrum() },
             'snare2': { 'num': 40, 'name': 'Electric Snare', 'obj': instrument.SnareDrum() },
             'sidestick': { 'num': 37, 'name': 'Side Stick', 'obj': instrument.SnareDrum() },
-            'clap': { 'num': 39, 'name': 'Hand Clap', 'obj': instrument.Percussion() },
+            'clap': { 'num': 39, 'name': 'Hand Clap', 'obj': instrument.SnareDrum() },
             'hihat1': { 'num': 42, 'name': 'Closed High Hat', 'obj': instrument.HiHatCymbal() },
             'hihat2': { 'num': 46, 'name': 'Open High Hat', 'obj': instrument.HiHatCymbal() },
             'hihat3': { 'num': 44, 'name': 'Pedal High Hat', 'obj': instrument.HiHatCymbal() },
@@ -187,8 +213,8 @@ class Drummer:
             'timbale2': { 'num': 66, 'name': 'Low Timbale', 'obj': instrument.Timbales() },
             'tambourine': { 'num': 54, 'name': 'Tambourine', 'obj': instrument.Tambourine() },
             'shaker': { 'num': 70, 'name': 'Maracas', 'obj': instrument.Maracas() },
-            'cabasa': { 'num': 69, 'name': 'Cabasa', 'obj': instrument.Percussion() },
-            'claves': { 'num': 75, 'name': 'Claves', 'obj': instrument.Percussion() },
+            'cabasa': { 'num': 69, 'name': 'Cabasa', 'obj': instrument.Maracas() },
+            'claves': { 'num': 75, 'name': 'Claves', 'obj': instrument.Woodblock() },
             'woodblock1': { 'num': 76, 'name': 'High Wood Block', 'obj': instrument.Woodblock() },
             'woodblock2': { 'num': 77, 'name': 'Low Wood Block', 'obj': instrument.Woodblock() },
             'triangle1': { 'num': 81, 'name': 'Open Triangle', 'obj': instrument.Triangle() },
@@ -196,5 +222,13 @@ class Drummer:
         }
         if key:
             return kit.get(key)
+        elif num:
+            for part in kit.values():
+                if part['num'] == num:
+                    return part
+        elif name:
+            for part in kit.values():
+                if part['name'] == name:
+                    return part
         else:
             return kit
\ No newline at end of file
